open Sys;;

type error_label =
    Lexical of string
  | Syntactical of string

exception Jakarta_error of error_label
exception End_ok;;

let nondef="_undefined"

(*type location = {ident_name:string ; pos:int}*)

type environment =((string*string) list) ref ;;

let (env:environment)   = ref []

(* initialization function *)
let init_env () =  env := [];;

(* accessor function *)
let get_env () = !env;;

(* printing function*)
let print_env () =
 let envi =  get_env () in
 let rec aux e =
  match e with  
     [] -> ()
   |  el::l ->  
       (print_string ("( "^(fst el)^" , "^(snd el)^" )\n")); 
       (aux l)
 in 
 aux envi 
;;

(* increment function *)
let add_env (st:string) (posfile:string) = 
  (env := (st,posfile)::(!env));;


type location =((string*int) list) ref ;;

let (loc:location)   = ref []

(* initialization function *)
let init_loc () =  loc := [];;

(* accessor function *)
let get_loc () = !loc;;

(* printing function*)
let print_loc () =
 let envi =  get_loc () in
 let rec aux e =
  match e with  
     [] -> ()
   |  el::l ->  
       (print_string ("( "^(fst el)^" , "^string_of_int (snd el)^" )\n")); 
       (aux l)
 in
 aux envi
;;

(* increment function *)
let add_loc (st:string) (pos:int) = 
  (loc := (st,pos)::(!loc));;



(* TAG CONTAINING THE NAME OF THE CURRENT SCANNED FILE *)

let current_filename: string ref = ref nondef;; 

(* initialization function *)
let init_current_filename () =  current_filename := nondef;;

(* increment function *)
let set_current_filename (st:string) = current_filename := st;;

(* accessor function for the current filename *)
let get_current_filename () = !current_filename;;

(* TAG CONTAINING THE NAME OF THE OUTPUT FILE *)

let outputfile : out_channel ref = ref stdout;;

(* initialization function *)
let init_outputfile () = outputfile := stdout;;

(* set function *)
let set_outputfile (st:string) = 
 if file_exists st then
  prerr_string 
   ("the file "^st^ " already exists and its contents will be deleted \n");
  flush stderr;
  outputfile := open_out (st) ;;

(* accessor function *)
let current_outputfile () = !outputfile;;


let file_output  name =
  let nome = String.sub name  0 ((String.length name)-3) in
  let oc = current_outputfile () in
  let mesg = ("F"^nome^"\n" )in
  let _= output_string oc mesg in
  flush oc;;

let ident_output  posident posfile identname =
  let oc = current_outputfile () in
  let mesg = ("R"^(string_of_int posident)^" "^posfile^"."^identname^"\n" )in
  let _= output_string oc mesg in
  flush oc;;

let produce_output () =
  let (envi:(string*string) list) = get_env () in
  let (to_add:(string*int) list) = List.rev (get_loc ()) in
  let rec aux li envir =
    match li with
      | [] -> ()
      | (st,pos)::lli ->
	  try 
	    let flname = List.assoc st envir in
     let _ = (ident_output pos flname st) in
     (aux lli envir)
	  with Not_found -> aux lli envir
  in
    aux to_add envi
;;


 (* %%% Local Variables: *)
 (* %%% igrep-files-var: "jakref_basic.ml jakref_lexer.mll jakref_parser.mly jakref.ml" *)
 (* %%% compile-command: "make -C .. jakref_basics.cmo" *)
 (* %%% indent-tabs-mode: nil *)
 (* %%% tab-width: 1 *)
 (* %%% indent-tabs-mode: nil *)
 (* %%% indent-tabs-mode: nil *)
 (* %%% tuareg-default-indent:1 *)
 (* %%% tuareg-begin-indent:1 *)
 (* %%% tuareg-let-indent:1 *)
 (* %%% tuareg-match-indent:-1 *)
 (* %%% tuareg-try-indent:1 *)
 (* %%% tuareg-with-indent:1 *)
 (* %%% tuareg-rule-indent:1 *)
 (* %%% fill-column: 85 *)
 (* %%% End: *)


 
