(*

Lexical Analysis

*)


{
  open List;;
  open Hashtbl;;
  open Jakref_basics;;
  open Jakref_parser;;
  
  let ignore_lex lb = ignore (Lexing.lexeme lb) ;;

  let manage_comment = ignore_lex ;;

  let lex_error st = ();;



  let symbol_table =Hashtbl.create 53;; 
   let _ = List.iter (fun (kwd,tok) -> Hashtbl.add symbol_table kwd tok)
	     [
	       ("="                , IS) ;
	       (","                , COMMA );
	       (";"                , SEMCOL );
	       ("{"                , LEFT_BRACE );
	       ("}"                , RIGHT_BRACE );
	       ("["                , LEFT_BRACKET );
	       ("]"                , RIGHT_BRACKET );
	       ("("                , LEFT_PAR );
	       (")"                , RIGHT_PAR );
	       ("<"                , LEFT_DEL );
	       (">"                , RIGHT_DEL );
	       (":"                , COLON );
	       ("."                , DOT );
	       ("|"                , CONST_SEP)
(* 	       ("\""                , STRING_DELIM) *)
	     ];;

 let keyword_table =Hashtbl.create 53;;
 let _ = List.iter (fun (kwd,tok) -> Hashtbl.add keyword_table kwd tok)
	   [("type", TYPE );
	    ("definition",DEFINITION);
	    ("record",RECORD);
	    ("variable",VARIABLE);
	    ("declaration",DECLARATION);
	    ("data",DATA);
	    ("with",WITH);
	    ("function",FUNCTION);
	    ("and",AND);
	    ("abstract",ABSTRACT);
	    ("import",IMPORT);
	    ("from",FROM);
	    ("into",INTO);
	    ("log",LOG);
	    ("lemmas",LEMMAS);
	    ("coqcommand",COQCOMMAND);
	    ("transparent",TRANSPARENT);
	    ("addprecond",ADDPRECOND);
	    ("opaque",OPAQUE);
	    ("extract",EXTRACT);
	    ("prefix",PREFIX);
	    ("coq",COQ);
	    ("pvs",PVS);
	    ("isa",ISA);
	    ("replace",REPLACE);
	    ("replace_type",REPLACETYPE);
	    ("reject",REJECT);
	    ("remove",REMOVE);
	    ("redundant",REDONDANT);
 	    ("dundant",DONDANT);
	    ("delete",DELETE);
	    ("substitute",SUBSTITUTE);
	    ("reduce",REDUCE);
	    ("reduce_after",REDUCEAFTER);
	    ("eliminate",ELIMINATE);
	    ("rules",RULES);
	    ("by",BY);
	    ("inside",INSIDE);
	    ("at",AT);
	    ("use",USE);
	    ("where",WHERE);
	    ("coercion",COERCION);
	    ("using",USING);
	    ("in",IN);
	    ("totalize",TOTALIZE);
	    ("determine",DETERMINE);
	    ("change",CHANGE);
	    ("to",TO);
	    ("conclusion",CONCLUSION);
	    ("prefer",PREFER);
	    ("long",LONG);
	    ("short",SHORT);
	    ("general",GENERAL);
	    ("instance",INSTANCE);
	    ("iflonggeneral",IFLGENERAL);
	    ("iflonginstance",IFLINSTANCE);
	    ("ifshortgeneral",IFSGENERAL);
	    ("ifshortinstance",IFSINSTANCE);
	    ("forget",FORGET);
	    ("print",PRINT);
	    ("drop",DROP);
	    ("select",SELECT);
	    ("sprint",SPRINT);
	    ("quit",QUIT)]

}

let blank = [' ' '\t']
let lowercase = ['a'-'z' ''-'' ''-'']
let uppercase = ['A'-'Z' ''-''  ''-'']
let identchar = 
  ['A'-'Z' 'a'-'z'  '0'-'9' ''-''  ''-''   ''-'' ''-'' '\''  '_']
let symbolchar =
  ['!' '$' '%' '*' '+' '-' '.' '/' ':' '<' '=' '>' '?' '^' '|' '~']

let specialsymbol =
  ['='',' ';' '{' '}' '[' ']' '(' ')' '<' '>' ':' '.' '|' ]

let decimal_literal = ['0'-'9']+

rule next_token = parse
    "(*"               { ignore_lex lexbuf ; comment lexbuf ; next_token lexbuf }
  | "*)"               { raise (Jakarta_error (Lexical "unbalanced comments")) }
  | '.'blank           {DOT_SEP } 
  | ".\n"              {DOT_SEP}
  | '\n'               {next_token lexbuf }
  | '.'eof             {DOT_SEP}
  | eof                { raise End_ok }
  | blank +            { next_token lexbuf }
  | "@"                { AROBASE }
  | "&"                { ESPERUET }
  | ":="               { ASSIGN }
  | "=="               { EQUALS }
  | "->"               { ARROW }
  | "=>"               { DARROW } 
  | "#forall"          { FORALL}
  | "#true"            { TRUE }
  | "#false"           { FALSE }
  | "#not"             { NOT }
  | '-'                { TIRET }
  | decimal_literal   {let id = Lexing.lexeme lexbuf in NUMBER(int_of_string(id))}
  | specialsymbol { 
		let id = Lexing.lexeme lexbuf in
		try find symbol_table id 
		with Not_found -> (lex_error id; next_token lexbuf)}
  | (lowercase|uppercase) identchar *  
    { 
		let id = Lexing.lexeme lexbuf in
		try find keyword_table id 
		with Not_found -> 
		  (
		    IDENT(id,(Lexing.lexeme_start lexbuf))
		  )
    }
  | '\'' (lowercase|uppercase) identchar * { TVIDENT(Lexing.lexeme lexbuf) }
  | '"' [^'"']* '"'
		{ 
		  let aux = Lexing.lexeme lexbuf in 
		  STRING (String.sub aux 1 (String.length aux - 2))
		}
  | '`'               { ECHAPEXPR }
  | _                  {lex_error (Lexing.lexeme lexbuf); next_token lexbuf } 

and comment = parse 
    "(*"    { ignore_lex lexbuf ; comment lexbuf }
  | "*)"    { ignore_lex lexbuf  }  
  | _       { manage_comment lexbuf ; comment lexbuf }
  | eof     { raise (Jakarta_error (Lexical "unbalanced comments")) }


 (* %%% Local Variables: *)
 (* %%% igrep-files-var: "jakref_basic.ml jakref_lexer.mll jakref_parser.mly jakref.ml" *)
 (* %%% compile-command: "make -C .. jakref_lexer.cmo" *)
 (* %%% indent-tabs-mode: nil *)
 (* %%% tab-width: 1 *)
 (* %%% indent-tabs-mode: nil *)
 (* %%% indent-tabs-mode: nil *)
 (* %%% tuareg-default-indent:1 *)
 (* %%% tuareg-begin-indent:1 *)
 (* %%% tuareg-let-indent:1 *)
 (* %%% tuareg-match-indent:-1 *)
 (* %%% tuareg-try-indent:1 *)
 (* %%% tuareg-with-indent:1 *)
 (* %%% tuareg-rule-indent:1 *)
 (* %%% fill-column: 85 *)
 (* %%% End: *)

