/*

  Synctactic Analysis

*/


%{

open List;;
open Jakref_basics;;


%}


/* Terminal Symbols (tokens). */

%token <string> STRING
%token <string*int> IDENT
%token <string> TVIDENT
%token <int> NUMBER
%token COMMA SEMCOL ECHAPEXPR
%token ASSIGN COLON 
%token DATA RECORD TYPE
%token IS WITH
%token LEFT_BRACE RIGHT_BRACE
%token LEFT_BRACKET RIGHT_BRACKET
%token LEFT_PAR RIGHT_PAR
%token LEFT_DEL RIGHT_DEL
%token ARROW DARROW
%token DOT DOT_SEP
%token CONST_SEP
%token TIRET
%token EQUALS
%token EOF
%token FUNCTION DEFINITION VARIABLE DECLARATION
%token AND
%token AT INSIDE IN BY REPLACE REPLACETYPE LOG REJECT 
%token REMOVE DELETE SUBSTITUTE REDUCE REDUCEAFTER DROP SELECT
%token PREFER FORGET 
%token SHORT LONG GENERAL INSTANCE 
%token IFLGENERAL IFSGENERAL IFLINSTANCE IFSINSTANCE
%token ABSTRACT INTO PREFIX
%token USE WHERE
%token COERCION USING
%token SPRINT PRINT QUIT EXTRACT
%token TOTALIZE DETERMINE
%token CHANGE CONCLUSION TO
%token ELIMINATE RULES
%token IMPORT FROM
%token COQ
%token PVS
%token ISA
%token AROBASE ESPERUET LEMMAS COQCOMMAND
%token TRANSPARENT OPAQUE ADDPRECOND REDONDANT DONDANT
%token FORALL TRUE FALSE NOT

/*%token TOTALIZE*/
/*%token MUTFUN MUTTYPE*/
/*%token <string> CIDENT*/
/*%token BEGIN END OF*/



/* Precedences and associativities. Lower precedences come first. */

/* expressions (type expr included) with parenthesis */
%nonassoc parenthesis  

/* expressions (type expr included) without parenthesis */           
%nonassoc application             

%left EQUALS 
/* %nonassoc LEFT_BRACE RIGHT_BRACE */
%left WITH
%left DOT

/* Axioms */

%start command; 
%type <unit> command

%%

/* Grammar Rules */

/* Entry rule of the Jakarta grammar */

command : 
declaration DOT_SEP {}  
 | QUIT DOT_SEP {}
 | command_statement DOT_SEP {}
 | error {}
 | error DOT_SEP {}
;

prefix_decl :
/*empty*/ {}
| PREFIX IDENT {}
;


/* Jakarta Commands */

command_statement : ABSTRACT IDENT with_item optional_and_item 
 prefix_decl options optional_filespec optional_logfilespec
 optional_lemmas optional_coqcommand
 {}

| EXTRACT optcoq optional_filespec 
  {}

| EXTRACT optcoq IDENT optional_filespec 
  {}

| IMPORT FROM IDENT 
  {}

| PRINT IDENT optional_filespec 
  {}

| PRINT optional_filespec 
  {}

| SPRINT optional_filespec 
  {}

| ABSTRACT IDENT error 
  {}

;




/* ****************************************************** */
/* ***************  Start of new parser *****************  */
/* ****************************************************** */


options : 
/*empty*/ 
    {}
  | options option {}
;

option:
       ADDPRECOND LEFT_DEL lemma RIGHT_DEL TO IDENT
         {}
     | TRANSPARENT simple_ident_list 
         {}

     | OPAQUE simple_ident_list 
       {}

     |  IN listloc DELETE listloc 
 {}

| DELETE listloc
  {}

| DELETE LEFT_BRACKET redondopt RIGHT_BRACKET listruleloc  
  {}

| REPLACE listloc BY listloc
  {}

| IN listloc REPLACE listloc BY listloc
  {}

| REPLACETYPE listargloc BY typeexpr
  {}

| SELECT listargloc
  {}

| IN listloc SELECT listargloc
  {}

| DROP listargloc
  {}

| DROP listfutureargloc
 {}

| DETERMINE simple_ident_list {}
| TOTALIZE simple_ident_list {}

| IN listloc REJECT non_empty_listofexpr 
  {}
| REJECT non_empty_listofexpr { }
| REMOVE LEFT_BRACKET redondopt RIGHT_BRACKET non_empty_listofexpr 
    {}
| IN listfutureconcl DELETE  non_empty_listofexpr
  {} 
| IN listfuturefun REMOVE LEFT_BRACKET redondopt RIGHT_BRACKET non_empty_listofexpr
  {}
| COERCION convert_list {}
| IN listloc COERCION convert_list 
    {}
;


redondopt: 
    DONDANT   {} 
| REDONDANT     {}
;

listruleloc: listruleloc ruleloc {}
  | ruleloc {}
;

ruleloc: IDENT DOT listint {}
;

listfutureconcl: futureconcl {}
  | listfutureconcl futureconcl {}
;

futureconcl: futurefun DOT subloc {}
;

listfuturefun: futurefun          {}
  | listfuturefun futurefun       {}
;

futurefun:  ESPERUET IDENT {}
;

listfutureargloc: futureargloc {}
  | listfutureargloc futureargloc {}
;

futureargloc: futurefun AROBASE listint
  {};

listloc: loc {}
  | listloc loc {};



loc:IDENT                     {}
  | IDENT DOT subloc          {}
  | ECHAPEXPR IDENT           {}  
  | LEFT_PAR c_expr RIGHT_PAR {}
;

subloc: listint DOT listint DOT listint {};

listargloc: listargloc argloc {} 
  | argloc {};

argloc: IDENT AROBASE listint {};

listint: NUMBER                        {}
  | LEFT_BRACE lint RIGHT_BRACE {};

lint: numbers          {}
  | lint COMMA numbers {}; 


/* ****************************************************** */
/* End of the new directive parser, nothing changed below
/* except the Fld attribute in  the entry s_expr  */
/* ****************************************************** */




optcoq:
    /* empty */                                 { "caml" }
  | LEFT_BRACKET COQ RIGHT_BRACKET              { "coq" }
  | LEFT_BRACKET PVS RIGHT_BRACKET              { "pvs" }
  | LEFT_BRACKET ISA RIGHT_BRACKET              { "isa" }


with_item:
  WITH ident_list {};

optional_and_item : 
/*empty*/ {}
| AND ident_list {};
;


simple_ident_list : 
  IDENT {}
| simple_ident_list IDENT {};


ident_list:
  IDENT {}
| ident_list IDENT {}
;





optional_filespec : 
/*empty*/ {}
| INTO IDENT {}
;

optional_logfilespec : 
/*empty*/ {}
| LOG IDENT {}
;

optional_lemmas:
/*empty*/ {}
| LEMMAS IDENT {}
;

optional_coqcommand:
/*empty*/ {}
| COQCOMMAND STRING {}
;


/********************************************/
/* local, general and global jtk directives */
/********************************************/




/********************************************/
/*    auxiliary rules for jtk directives    */
/********************************************/


/*
item_replace:
 expr BY expr {}
;
*/

 convert_list:  
 IDENT {}
 | convert_list IDENT  {}
;




/*
number_list:
  number_list  numbers{}
| numbers {}
;
*/


numbers:
  NUMBER {}
| NUMBER TIRET NUMBER {}
;


/* Jakarta Declaration */

declaration :
  types_declaration {}
| functions_declaration {}
;


/* HEAD PART DEFINITION */ 

headrec    : RECORD IDENT        
 {
  let fname =  get_current_filename () in 
    add_env (fst $2) fname
 } 
;

headdata   : DATA IDENT          {
  let fname =  get_current_filename () in 
    add_env (fst $2) fname
} 
;

headfunc   : FUNCTION IDENT      {
  let fname =  get_current_filename () in 
    add_env (fst $2) fname} 
;

headdef    : DEFINITION IDENT    {
  let fname =  get_current_filename () in 
    add_env (fst $2) fname} 
;

headvar    : VARIABLE IDENT      {
  let fname =  get_current_filename () in 
    add_env (fst $2) fname} 
;

headdecl   : DECLARATION IDENT   {
  let fname =  get_current_filename () in 
    add_env (fst $2) fname} 
;


/* types definition rules */

types_declaration :
  list_of_simple_types_declaration {}
| type_declaration {}
;
 
list_of_simple_types_declaration:
  simple_type_declaration {}
| list_of_simple_types_declaration AND simple_type_declaration {}


type_declaration:
  TYPE dectype {}
| TYPE IDENT IS typeexpr 
      {
  let fname =  get_current_filename () in 
    add_env (fst $2) fname}  
| TYPE error 
	  {}
;
 

simple_type_declaration:
  record_declaration {}
| data_declaration {}
;


record_declaration:
 headrec IS  
  LEFT_BRACE listoffields RIGHT_BRACE 
  {}
| headrec error 
	  {}
;


data_declaration:
headdata IS listofconstructors {}
| headdata error 
	  {}
;



dectype : 
  IDENT varlist {
  let fname =  get_current_filename () in 
    add_env (fst $1) fname}
| LEFT_PAR dectype RIGHT_PAR  {}
| IDENT error {}
;

varlist : 
/*empty list */ {}
|varlist TVIDENT  {}
;


functions_declaration :
  list_of_functions_declaration 
  {}
| variable_declaration 
	{}
| declaration_declaration 
	{}
| definition_declaration 
		{}
;


list_of_functions_declaration:
  simple_function_declaration {}
| list_of_functions_declaration     
      AND simple_function_declaration {}
;
 
variable_declaration:
headvar COLON signature 
      {} 
| headvar error 
	  {}

;

declaration_declaration:
  headdecl COLON signature 
  {} 
| headdecl error 
      {}

;

definition_declaration:
  headdef COLON typeexpr ASSIGN expr 
  {}
| headdef error 
	  {}

;

simple_function_declaration:
  headfunc COLON signature ASSIGN 
  listofrules  
  {}
| headfunc error 
	  {}


;


/* FIELDS DEFINITION */

fields:
  field SEMCOL              
             {}
| fields  field SEMCOL
            {}
;


listoffields : 
  field {}
| fields field 
      {}
;

field : IDENT COLON typeexpr  
  {
  let fname =  get_current_filename () in 
    add_env (fst $1) fname} 
| error {}
;



/* CONSTRUCTORS DEFINITION */


constructor : 
  IDENT listoftypeexpr  
          {
           let fname =  get_current_filename () in 
             add_env (fst $1) fname
          } 
| error {}
;

constructors : 
  constructor  CONST_SEP                        
         {} 
| constructors constructor CONST_SEP
         {} 
;


listofconstructors:
  constructor {}
| constructors constructor 
      {}


/* RULES DEFINITION */

listofrules:
  completerule {}
| listofrules_with_semcol completerule {}
;

listofrules_with_semcol : 
  completerule SEMCOL              
             {}
| listofrules_with_semcol completerule SEMCOL  
             {}
;


completerule : 
  LEFT_DEL IDENT RIGHT_DEL rule  {}
| rule            {}
| error          {} 
;


rule : 
  DARROW atomicrule                   
            {} 
| listofatomicrules DARROW atomicrule 
            {} ; 

atomicrules : 
  atomicrule  COMMA                       
           {}
| atomicrules  atomicrule COMMA 
           {} 
;

listofatomicrules :
  atomicrule {}
| atomicrules atomicrule {}
;


atomicrule : 
  expr ARROW expr  
           {}
| error   {};



/* EXPRESSIONS */

/* ASSIGN  DEFINITION */


fdassigns:   
  fdassign SEMCOL {}
| fdassigns fdassign SEMCOL {}
;


listoffdassigns :  
  fdassign  {}
| fdassigns fdassign {}
;


fdassign : IDENT ASSIGN expr 
        {add_loc (fst $1) (snd $1)}
| error {}
 ;

/* ATRIBUTION  DEFINITION */


fdatribs:   
  fdatrib SEMCOL {}
| fdatribs fdatrib SEMCOL {}
;


listoffdatribs :  
  fdatrib  {}
| fdatribs fdatrib {} 
;


fdatrib : IDENT IS expr 
        {add_loc (fst $1) (snd $1)}
| error {}
 ;


/* EXPRESSION DEFINITIONS */

listofexpr:
/*empty*/ {}
| listofexpr s_expr
        {}
| listofexpr LEFT_PAR c_expr RIGHT_PAR
        {}
;

non_empty_listofexpr:
  listofexpr s_expr
        {}
| listofexpr LEFT_PAR c_expr RIGHT_PAR 
        {}
;

s_expr:
  IDENT  {add_loc (fst $1) (snd $1)} 
| s_expr DOT IDENT {add_loc (fst $3) (snd $3)}
| LEFT_PAR c_expr RIGHT_PAR DOT IDENT {add_loc (fst $5) (snd $5)}
| LEFT_BRACE listoffdatribs RIGHT_BRACE  
	 {}
| LEFT_PAR s_expr RIGHT_PAR {}
;


c_expr:
  IDENT  non_empty_listofexpr       
  {add_loc (fst $1) (snd $1)}
| s_expr EQUALS s_expr                
      {}
| s_expr EQUALS LEFT_PAR c_expr RIGHT_PAR                
      {}
| LEFT_PAR c_expr RIGHT_PAR EQUALS s_expr                
      {}
| LEFT_PAR c_expr RIGHT_PAR  EQUALS LEFT_PAR c_expr RIGHT_PAR                
      {}
| s_expr WITH LEFT_BRACE listoffdassigns RIGHT_BRACE
	      {} 
| LEFT_PAR c_expr RIGHT_PAR  WITH LEFT_BRACE listoffdassigns RIGHT_BRACE
	      {}
| LEFT_PAR c_expr RIGHT_PAR {} 

;


expr :
  s_expr {}
| c_expr {}
| error {}

;



/* UNIVERSE EXPRESIONS DEFINITION */



typeexpr : 
  st_typeexpr {}
| ct_typeexpr {}
| error {}
;

listoftypeexpr :
/*empty */                {}
| listoftypeexpr LEFT_PAR ct_typeexpr RIGHT_PAR {}
| listoftypeexpr st_typeexpr {}
;


non_empty_listoftypeexpr :
 listoftypeexpr LEFT_PAR ct_typeexpr RIGHT_PAR {}
| listoftypeexpr st_typeexpr {}
;


st_typeexpr:
  TVIDENT {}
| IDENT   {add_loc (fst $1) (snd $1)}
| LEFT_PAR st_typeexpr RIGHT_PAR {}
;

ct_typeexpr :  
  IDENT non_empty_listoftypeexpr {add_loc (fst $1) (snd $1)}
| LEFT_PAR ct_typeexpr RIGHT_PAR {}
;

/* SIGNATURE DEFINITION */

signature:
  typeexpr {}
|  c_signature {}
;
 
c_signature : 
arrowlist typeexpr {};

typefun:
  LEFT_PAR c_signature RIGHT_PAR {}
| LEFT_PAR typefun RIGHT_PAR {};


arrowlist : 
  typeexpr  ARROW {}
| typefun   ARROW {}
| arrowlist typeexpr ARROW {} 
| arrowlist typefun  ARROW {}
;



lemma:
 FORALL IDENT COLON signature COMMA lemma 
 {}
| lemma ARROW lemma {}
| lemma AND lemma {}
| term IS term {}
| NOT lemma {}
| FALSE {}
| TRUE {}
| term {}
| LEFT_PAR lemma RIGHT_PAR {}
;

term:
 IDENT lterm {}
| IDENT {}
| ECHAPEXPR lemma ECHAPEXPR {}
| LEFT_PAR term RIGHT_PAR {}
;

lterm:
 lterm term_no_appl {}
| term_no_appl {}
;

term_no_appl:
 IDENT {}
| ECHAPEXPR lemma ECHAPEXPR {}
| LEFT_PAR term RIGHT_PAR {}
;

/*
 *** Local Variables: ***
 *** igrep-files-var: "jakref_lexer.mll jakref_parser.mly jakref.ml" ***
 *** compile-command: "make -C .. jakref_parser.cmo" ***
 *** indent-tabs-mode: nil ***
 *** tab-width: 1 ***
 *** indent-tabs-mode: nil ***
 *** indent-tabs-mode: nil ***
 *** tuareg-default-indent:1 ***
 *** tuareg-begin-indent:1 ***
 *** tuareg-let-indent:1 ***
 *** tuareg-match-indent:-1 ***
 *** tuareg-try-indent:1 ***
 *** tuareg-with-indent:1 ***
 *** tuareg-rule-indent:1 ***
 *** fill-column: 85 ***
 *** End: ***
*/

